import httpx
import asyncio
from typing import Optional
import webbrowser


API_BASE = "https://api.vercel.com"


class Vercel:
    def __init__(self, bearer_token: str, team_id: Optional[str] = None):
        self.bearer_token = bearer_token
        self.team_id = team_id
        self._deployments = {}

    async def deployments(
        self,
        *,
        state: Optional[str] = None,
        target: Optional[str] = None,
    ):
        url = f"{API_BASE}/v6/deployments"
        headers = {
            "Authorization": f"Bearer {self.bearer_token}",
            "Content-Type": "application/json",
        }
        params = {
            "teamId": self.team_id,
            "state": state,
            "target": target,
            "limit": 100,
        }

        self._deployments = {}
        async with httpx.AsyncClient() as client:
            request = client.build_request("GET", url, headers=headers, params=params)
            print(f"REQUEST: {request.method} {request.url}")

            response = await client.send(request)
            json = response.json()
            print(f"RESPONSE: {response.status_code} {json}")

            for deployment in json["deployments"]:
                self._deployments[deployment["uid"]] = deployment

            return self._deployments

    async def cancel_deployments(
        self,
        deployment_ids: list[str],
    ):
        headers = {
            "Authorization": f"Bearer {self.bearer_token}",
            "Content-Type": "application/json",
        }
        params = {
            "teamId": self.team_id,
        }

        responses = []
        async with httpx.AsyncClient() as client:
            requests = [
                client.build_request(
                    "PATCH",
                    f"{API_BASE}/v12/deployments/{deployment_id}/cancel",
                    headers=headers,
                    params=params,
                )
                for deployment_id in deployment_ids
            ]

            for request in requests:
                print(f"REQUEST: {request.method} {request.url}")

            responses.extend(
                await asyncio.gather(
                    *[client.send(request) for request in requests],
                    return_exceptions=True,
                )
            )

            for response in responses:
                print(f"RESPONSE: {response.status_code} {response.json()}")

        return all(r.status_code == 200 for r in responses)

    def open_deployment(self, deployment_id: str):
        deployment = self._deployments[deployment_id]
        webbrowser.open(deployment["inspectorUrl"])



from textual import work, on
from textual.app import App, ComposeResult
from textual.widgets import Header, Footer, LoadingIndicator, DataTable
from rich.text import Text
from vercel_mgmt.vercel import Vercel
import argparse
import humanize
from datetime import datetime


class VercelMGMT(App):
    TITLE = "Vercel MGMT"
    SUB_TITLE = "Non-production builds"
    BINDINGS = [
        ("q", "quit", "Quit"),
        ("r", "refresh", "Refresh"),
        ("space", "open", "Open Deployment"),
        ("c", "cancel", "Cancel Selected Deployments"),
    ]

    def __init__(self, vercel: Vercel):
        super().__init__()
        self.vercel = vercel
        self.selected_deployments = set()

    def compose(self) -> ComposeResult:
        yield Header()
        yield LoadingIndicator()
        yield DataTable()
        yield Footer(show_command_palette=False)

    def on_mount(self) -> None:
        self.create_table()
        self.load_deployments()

    def create_table(self) -> None:
        table = self.query_one(DataTable)
        table.cursor_type = "row"

        table.add_column("", key="selected")
        table.add_column("created", key="created")
        table.add_column("state", key="state")
        table.add_column("project", key="project")
        table.add_column("creator", key="creator")
        table.add_column("branch", key="branch")
        table.add_column("commit", key="commit")

    def action_cancel(self) -> None:
        if not self.selected_deployments:
            return

        self.query_one(LoadingIndicator).display = True
        self.cancel_deployments()

    def action_refresh(self) -> None:
        self.query_one(LoadingIndicator).display = True
        self.selected_deployments.clear()
        self.load_deployments()

    def action_open(self) -> None:
        table = self.query_one(DataTable)
        if not len(table.rows):
            return
        row_key, _ = table.coordinate_to_cell_key(table.cursor_coordinate)
        self.vercel.open_deployment(row_key)

    @on(DataTable.RowSelected)
    def toggle_row_selection(self, event: DataTable.RowSelected) -> None:
        table = event.control
        row_key = event.row_key
        deployment_id = row_key.value
        if deployment_id in self.selected_deployments:
            self.selected_deployments.remove(deployment_id)
            table.update_cell(row_key, "selected", " ")
        else:
            self.selected_deployments.add(deployment_id)
            table.update_cell(row_key, "selected", "✔")

        row_idx, _ = table.cursor_coordinate
        if row_idx < len(table.rows) - 1:
            table.move_cursor(row=row_idx + 1)

    @work(exclusive=True)
    async def load_deployments(self) -> None:
        deployments = await self.vercel.deployments(
            state="QUEUED,BUILDING", target="preview"
        )
        self.query_one(LoadingIndicator).display = False
        table = self.query_one(DataTable)
        table.clear()
        for deployment_id, deployment in deployments.items():
            table.add_row(
                Text(" "),
                Text(
                    humanize.naturaltime(
                        datetime.fromtimestamp(int(deployment["created"]) / 1000)
                    ),
                    style="cyan",
                ),
                Text(
                    deployment["state"],
                    style="yellow" if deployment["state"] == "BUILDING" else None,
                ),
                Text(deployment["name"]),
                Text(deployment["creator"]["username"], style="italic green"),
                Text(deployment["meta"]["githubCommitRef"], style="lightblue"),
                Text(
                    deployment["meta"]["githubCommitMessage"][:50]
                    + (
                        "..."
                        if len(deployment["meta"]["githubCommitMessage"]) > 50
                        else ""
                    ),
                ),
                key=deployment_id,
            )

    @work(exclusive=True)
    async def cancel_deployments(self) -> None:
        deployment_ids = list(self.selected_deployments)
        await self.vercel.cancel_deployments(deployment_ids)
        self.selected_deployments.clear()
        self.load_deployments()


def main():
    parser = argparse.ArgumentParser(description="Vercel Management Tool")
    parser.add_argument("--token", "-t", required=True, help="Vercel bearer token")
    parser.add_argument("--team-id", "-tid", help="Vercel team ID (optional)")
    args = parser.parse_args()

    vercel = Vercel(args.token, args.team_id)
    mgmt = VercelMGMT(vercel)
    mgmt.run()


if __name__ == "__main__":
    main()
